import os

import yaml
import click
import pandas as pd
from typing import Type, List
from bullet import Bullet, Input, YesNo

from ankimaker.config import Config, FilterConfig


__CONFIRMATION_QUESTION = """
Is this read option {option} correct?
______________
{preview}
"""

__SUCCESS_MESSAGE = """
All done. You can now use your configuration with this  command:
{command}
"""

__COMMAND_SAMPLE = """ankimaker csv \
-i {input} \
-o my-new-deck.apkg \
--conf {output}
"""


__ADD_FILTER_QUESTION = """Do you want do add a filter to the configuration?"""


def create_config(input_file, output_path):

    separators = handle_read_option(
        input_file, read_option='sep', sep=','
    )
    header = handle_read_option(
        input_file, read_option='header', header=None,
        sep=separators, option_type=int
    )

    question_column = get_column('question')
    answer_column = get_column('answer')

    filters = process_filters(input_file, header, separators)

    new_config = Config(
        separators=separators,
        header=header,
        question_column=question_column,
        answer_column=answer_column,
        filters=filters
    )
    save_file(new_config, output_path)

    finish_message = __SUCCESS_MESSAGE.format(command=make_sample_command(input_file, output_path))
    click.clear()
    click.echo(finish_message)


def process_filters(input_file, header, separators):
    df = pd.read_csv(input_file, header=header, sep=separators)
    filters = add_filters_to_config(df)
    return filters


def __inline_yes_or_no_question(question):
    answer = YesNo(prompt=question, default='n').launch()
    return answer


def add_filters_to_config(df: pd.DataFrame) -> List[List[FilterConfig]]:
    config = Config()
    should_add_filter = __inline_yes_or_no_question(__ADD_FILTER_QUESTION)
    while should_add_filter:
        config = add_filter_to_or_create_filter_group(df, config)
        should_add_filter = __inline_yes_or_no_question(__ADD_FILTER_QUESTION)
    return config.filters


def add_filter_to_or_create_filter_group(df: pd.DataFrame, config: Config) -> Config:
    config_has_filters = len(config.filters) > 0
    chosen_group = -1
    if config_has_filters:
        filter_options = [f'({"|".join(map(str, group)):.45s})' for group in config.filters]
        filter_options = [f'Group{i+1}{s}' for i, s in enumerate(filter_options)]
        cli = Bullet(
            prompt="Select group: ",
            choices=["Create new", *filter_options],
            return_index=True,
        )
        chosen_group = cli.launch()[1] - 1
    new_filter = create_filter_config(df)
    if chosen_group < 0:
        config.filters.append([new_filter])
    else:
        config.filters[chosen_group].append(new_filter)
    return config


def create_filter_config(df: pd.DataFrame) -> FilterConfig:
    options = list(df.columns)
    cli = Bullet(
        prompt="Select a columns to filter: ",
        choices=list(map(str, options)),
        return_index=True
    )
    chosen = cli.launch()[1]
    filter_column = options[chosen]
    columns_values = df[filter_column].unique()
    values = Input(f'Which values fo filter out? values[{columns_values}]: ').launch()
    new_filter = FilterConfig(column=filter_column, values=values)
    return new_filter


def get_column(name: str) -> str:
    answer = click.prompt(f'Which is your {name} column?', type=str, confirmation_prompt=True)
    return answer


def handle_read_option(input_file, read_option, option_type: Type = str, **kargs):
    preview: str
    is_finished = False
    while not is_finished:
        preview = load_preview(input_file, **kargs)
        question = __CONFIRMATION_QUESTION.format(
            option=str(read_option),
            preview=preview
        )
        answer: str = click.prompt(question, type=str, default='y')
        click.clear()
        if answer.lower()[0] == 'y':
            return kargs[read_option]
        else:
            updated_option: str = click.prompt('Insert the correct', type=option_type)
            kargs[read_option] = updated_option
            click.clear()


def load_preview(input_file, *args, **kargs):
    try:
        df = pd.read_csv(input_file, *args, **kargs)
        preview = df.head()
    except pd.errors.ParserError:
        preview = 'FAIL'
    return preview


def save_file(config: Config, file_path):
    if '~' in file_path:
        file_path = os.path.expanduser(file_path)
    with open(file_path, 'w') as f:
        yaml.dump(config, f)


def make_sample_command(input_config, output):
    command = __COMMAND_SAMPLE.format(
        input=input_config, output=output
    )
    return command
